//
//		firmware for Stopping Circuit
//			another version by ZakLab
//
//       2017/08/20 by ZakLab
//			 all rights reserved
//

#include <TimerOne.h>


#define kLED_Green				7				// D7
#define	kLED_Orange				4				// D4
#define	kSmallLED_Red			9				// D9
#define	kSmallLED_Green		10			// D10
#define	kSmallLED_Blue		11			// D11
#define	kBigLED_Red				3				// D3
#define	kBigLED_Green			5				// D5
#define	kBigLED_Blue			6				// D6
#define	kBattelyCheck			A0			// A0
#define	kSignal						2				// D2

#define	kMaxLEDValue			255
#define	kMaxLEDValue2			15
#define	kSignalLevelHigh	1800		// uSec


volatile int16_t	gGreenLEDValue = 0;
volatile int16_t	gOrangeLEDValue = 0;
volatile uint8_t	gSpecialMode = 0;

int32_t						gLimitVoltage = 0;


// LED update handler
void setLEDValue(uint8_t	pin, uint16_t	theValue)
{
	if (pin == kLED_Green)
	{
		gGreenLEDValue = kMaxLEDValue2 - (theValue >> 4);
	}
	else if (pin == kLED_Orange)
	{
		gOrangeLEDValue = kMaxLEDValue2 - (theValue >> 4);
	}
	else
	{
		analogWrite(pin, kMaxLEDValue - theValue);
	}
}


// software PWM for green and orange LED
// (TimerOne interrupt handler)
void morePWM()
{
	static int16_t	sLEDCounter = 0;
	
	// PWM for more 2 LED
	sLEDCounter++;
	if (sLEDCounter > kMaxLEDValue2)
	{
		sLEDCounter = 0;
		digitalWrite(kLED_Green, 1);
		digitalWrite(kLED_Orange, 1);
	}
	
	if (sLEDCounter > gGreenLEDValue)
	digitalWrite(kLED_Green, 0);
	
	if (sLEDCounter > gOrangeLEDValue)
	digitalWrite(kLED_Orange, 0);
}


// signal pulse handler
// (interrut handler)
void changeSignal(void)
{
	static uint32_t		sSignalStartTime = 0;
	
	if (digitalRead(kSignal))
	{
		sSignalStartTime = micros();
	}
	else
	{
		gSpecialMode =  ((micros() - sSignalStartTime) > kSignalLevelHigh);
	}
}


// chu-ni start up demo
void startupDemo(void)
{
	uint8_t					counter;
	
	// power on each LED
	setLEDValue(kLED_Green, 16);
	delay(200);
	setLEDValue(kLED_Green, 0);
	
	setLEDValue(kSmallLED_Red, 90);
	delay(150);
	setLEDValue(kSmallLED_Red, 0);
	
	setLEDValue(kSmallLED_Green, 80);
	delay(150);
	setLEDValue(kSmallLED_Green, 0);
	
	setLEDValue(kSmallLED_Blue, 180);
	delay(150);
	setLEDValue(kSmallLED_Blue, 0);
	
	setLEDValue(kLED_Orange, 16);
	delay(200);
	setLEDValue(kLED_Orange, 0);
	
	setLEDValue(kBigLED_Red, 127);
	delay(150);
	setLEDValue(kBigLED_Red, 0);
	
	setLEDValue(kBigLED_Green, 127);
	delay(150);
	setLEDValue(kBigLED_Green, 0);
	
	setLEDValue(kBigLED_Blue, 127);
	delay(150);
	setLEDValue(kBigLED_Blue, 0);	
	
	// demo LED
	for (counter = 10; counter > 0; counter--)
	{
		setLEDValue(kLED_Green, 16);
		setLEDValue(kLED_Orange, 0);
		
		if (counter & 0x01)
		{
			setLEDValue(kSmallLED_Red, 90);
			setLEDValue(kBigLED_Blue, 127);
		}
		else
		{
			setLEDValue(kSmallLED_Red, 0);
			setLEDValue(kBigLED_Blue, 0);	
		}
		
		if (counter & 0x02)
		{
			setLEDValue(kBigLED_Red, 127);
			setLEDValue(kSmallLED_Green, 90);
		}
		else
		{
			setLEDValue(kSmallLED_Green, 0);
			setLEDValue(kBigLED_Red, 0);
		}
		
		if (counter & 0x04)
		{
			setLEDValue(kSmallLED_Blue, 90);
			setLEDValue(kBigLED_Green, 127);
		}
		else
		{
			setLEDValue(kSmallLED_Blue, 0);
			setLEDValue(kBigLED_Green, 0);
		}
		
		delay(150);
		
		setLEDValue(kLED_Green, 0);
		setLEDValue(kLED_Orange, 16);
		
		delay(150);
	}
	
	setLEDValue(kLED_Green, 0);
	setLEDValue(kLED_Orange, 0);
	setLEDValue(kSmallLED_Red, 0);
	setLEDValue(kSmallLED_Green, 0);
	setLEDValue(kSmallLED_Blue, 0);
	setLEDValue(kBigLED_Red, 0);
	setLEDValue(kBigLED_Green, 0);
	setLEDValue(kBigLED_Blue, 0);
	
	delay(800);
	
	// initialize done
	setLEDValue(kLED_Green, 16);
	setLEDValue(kLED_Orange, 16);
	setLEDValue(kSmallLED_Red, 0);
	setLEDValue(kSmallLED_Green, 0);
	setLEDValue(kSmallLED_Blue, 0);
	setLEDValue(kBigLED_Red, 0);
	setLEDValue(kBigLED_Green, 0);
	setLEDValue(kBigLED_Blue, 0);
}


// default Arduino setup function
void setup()
{
	// LED port initialize
	setLEDValue(kLED_Green, 0);
	setLEDValue(kLED_Orange, 0);
	setLEDValue(kSmallLED_Red, 0);
	setLEDValue(kSmallLED_Green, 0);
	setLEDValue(kSmallLED_Blue, 0);
	setLEDValue(kBigLED_Red, 0);
	setLEDValue(kBigLED_Green, 0);
	setLEDValue(kBigLED_Blue, 0);
	
	pinMode(kLED_Green, OUTPUT);
	pinMode(kLED_Orange, OUTPUT);
	pinMode(kSmallLED_Red, OUTPUT);
	pinMode(kSmallLED_Green, OUTPUT);
	pinMode(kSmallLED_Blue, OUTPUT);
	pinMode(kBigLED_Red, OUTPUT);
	pinMode(kBigLED_Green, OUTPUT);
	pinMode(kBigLED_Blue, OUTPUT);
	
	// check port initaalize
	pinMode(kSignal, INPUT);
	
	// setup LED timer
	Timer1.initialize(100); 
	Timer1.attachInterrupt(morePWM); 
	
	// get first voltage
	gLimitVoltage = analogRead(kBattelyCheck);
	gLimitVoltage *= 85;
	gLimitVoltage /= 100;
	
	// setup signal
	pinMode(kSignal, INPUT);
	digitalWrite(kSignal, HIGH);
	
	// initialize done work
	startupDemo();
	
	// start to check interrupt
	attachInterrupt(0, changeSignal, CHANGE);
}


// update big full color LED
void setBigFullColorLED(int16_t		currentBattery)
{
	if (currentBattery > gLimitVoltage)
	{
		setLEDValue(kBigLED_Red, 0);
		setLEDValue(kBigLED_Green, 0);
		setLEDValue(kBigLED_Blue, 127);
	}
	else
	{
		setLEDValue(kBigLED_Red, 127);
		setLEDValue(kBigLED_Green, 0);
		setLEDValue(kBigLED_Blue, 0);
	}
}


// update small full color LED
void setSmallFullColorLED(int16_t		currentBattery)
{
	// over 11V				white					255, 255, 255
	// 9.0 ~ 10.9V		orange				255, 183, 076
	// 8.0 ~ 8.9V			yellow				255, 255, 000
	// 6.0 ~ 7.9V			green					000, 255, 000
	// 5.0 ~ 5.9V			light blue		000, 255, 255
	// under 5V				purple				000, 000, 255
	
	// base balance of RGB   90, 80, 180
	
	if (currentBattery > 751)					// over 11V
	{			// white
		setLEDValue(kSmallLED_Red, 60);
		setLEDValue(kSmallLED_Green,60);
		setLEDValue(kSmallLED_Blue, 60);
	}
	else if (currentBattery > 614)		// over 9V
	{			// orange
		setLEDValue(kSmallLED_Red, 80);
		setLEDValue(kSmallLED_Green, 15);
		setLEDValue(kSmallLED_Blue, 0);
	}
	else if (currentBattery > 546)		// over 8V
	{			// yellow
		setLEDValue(kSmallLED_Red, 45);
		setLEDValue(kSmallLED_Green, 40);
		setLEDValue(kSmallLED_Blue, 0);
	}
	else if (currentBattery > 410)		// over 6V
	{			// green
		setLEDValue(kSmallLED_Red, 0);
		setLEDValue(kSmallLED_Green, 80);
		setLEDValue(kSmallLED_Blue, 0);
	}
	else if (currentBattery > 341)		// over 5V
	{			// light blue
		setLEDValue(kSmallLED_Red, 0);
		setLEDValue(kSmallLED_Green, 60);
		setLEDValue(kSmallLED_Blue, 20);
	}
	else														// under 5V
	{			// purple
		setLEDValue(kSmallLED_Red, 90);
		setLEDValue(kSmallLED_Green, 0);
		setLEDValue(kSmallLED_Blue, 90);
	}
}


// Arduino default loop function
void loop()
{
	int		currentBattery;
	
	// Battery check
	currentBattery = analogRead(kBattelyCheck);
	
	if (! gSpecialMode)
	{
		// Set indicator LED
		setLEDValue(kLED_Green, 16);
		setLEDValue(kLED_Orange, 16);
		
		// Set Big LED
		setBigFullColorLED(currentBattery);
		
		// Set Small LED
		setSmallFullColorLED(currentBattery);
	}
	else
	{
		setLEDValue(kLED_Green, 0);
		setLEDValue(kLED_Orange, 0);
		setLEDValue(kSmallLED_Red, 0);
		setLEDValue(kSmallLED_Green, 0);
		setLEDValue(kSmallLED_Blue, 255);
		setLEDValue(kBigLED_Red, 0);
		setLEDValue(kBigLED_Green, 0);
		setLEDValue(kBigLED_Blue, 255);
	}
}


